INF_LITERAL = str(1E1000)
NAN_LITERAL = str(1E1000 / 1E1000)
TRUE_LITERAL = "true"
FALSE_LITERAL = "false"
NULL_LITERAL = "null"
INT_PATTERN = fr"[+-]?\d+"
FLOAT_PATTERN = fr"(?:[+-]?\d+(?:\.\d+)?(?:e[+-]?\d+)?|[+-]?{INF_LITERAL}\b|{NAN_LITERAL}\b)"
BOOL_PATTERN = fr"({TRUE_LITERAL}|{FALSE_LITERAL})\b"
STRING_PATTERN = fr"\"(?:(?:\\\")|[^\"])*\""
NULL_PATTERN = fr"\b{NULL_LITERAL}\b"
ELEMENTARY_TYPES_PATTERN = fr"{FLOAT_PATTERN}|{INT_PATTERN}|{BOOL_PATTERN}|{STRING_PATTERN}|{NULL_PATTERN}"
ARRAY_PATTERN = r"\[(?R)?(?:,(?R))*\]"
OBJECT_PATTERN = r"\{(?:(?R):(?R))?(?:,(?R):(?R))*\}"
VALUE_PATTERN = fr"\s*({ELEMENTARY_TYPES_PATTERN}|" + fr"{ARRAY_PATTERN}|{OBJECT_PATTERN})\s*"


import re
import regex
from kolyaklimkLab3.MySerializer import Serializer
from kolyaklimkLab3.MyParser.Parser import Parser
from kolyaklimkLab3.MyParser import nonetype
from kolyaklimkLab3.MyParser.JsonParser import TRUE_LITERAL, FALSE_LITERAL, NULL_LITERAL, INT_PATTERN, FLOAT_PATTERN, BOOL_PATTERN, \
    STRING_PATTERN, NULL_PATTERN, VALUE_PATTERN


class Json(Serializer):
    def dumps(self, obj) -> str:
        def dumps_from_dict(string) -> str:
            if type(string) is nonetype:
                return NULL_LITERAL

            if type(string) is bool:
                return TRUE_LITERAL if string else FALSE_LITERAL

            if type(string) is str:
                return '"' + string.replace('\\', "\\\\").replace('"', r"\"").replace("'", r"\'") + '"'

            if type(string) in (int, float):
                return str(string)

            if type(string) is list:
                return '[' + ", ".join([dumps_from_dict(item) for item in string]) + ']'

            if type(string) is dict:
                return '{' + ", ".join([f"{dumps_from_dict(item[0])}: "
                                        f"{dumps_from_dict(item[1])}" for item in string.items()]) + '}'
            else:
                raise ValueError

        obj = Parser.to_dict(obj)
        return dumps_from_dict(obj)

    def loads(self, obj: str):
        def loads_to_dict(string: str):
            string = string.strip()

            match = re.fullmatch(NULL_PATTERN, string)
            if match:
                return None

            match = re.fullmatch(BOOL_PATTERN, string)
            if match:
                return match.group(0) == TRUE_LITERAL

            match = re.fullmatch(INT_PATTERN, string)
            if match:
                return int(match.group(0))

            match = re.fullmatch(FLOAT_PATTERN, string)
            if match:
                return float(match.group(0))

            match = re.fullmatch(STRING_PATTERN, string)
            if match:
                ans = match.group(0)
                ans = ans.replace('\\\\', "\\").replace(r"\"", '"').replace(r"\'", "'")
                return ans[1:-1]

            if string[0] == '[' and string[-1] == ']':
                string = string[1:-1]
                matches = regex.findall(VALUE_PATTERN, string)
                return [loads_to_dict(match[0]) for match in matches]

            if string[0] == '{' and string[-1] == '}':
                string = string[1:-1]
                matches = regex.findall(VALUE_PATTERN, string)

                return {loads_to_dict(matches[i][0]): loads_to_dict(matches[i + 1][0]) for i in
                        range(0, len(matches), 2)}
            else:
                raise ValueError

        obj = loads_to_dict(obj)
        return Parser.from_dict(obj)



import inspect
from kolyaklimkLab3.MyParser import nonetype, moduletype, codetype, celltype, \
    functype, smethodtype, cmethodtype, CODE_PROPS, UNIQUE_TYPES, \
    TYPE_KW, SOURCE_KW, CODE_KW, GLOBALS_KW, NAME_KW, DEFAULTS_KW, CLOSURE_KW, \
    BASES_KW, DICT_KW, CLASS_KW, OBJECT_KW


class Parser:
    @classmethod
    def to_dict(cls, obj, is_inner_func=False):
        def get_obj_dict(obj_dict):
            dct = {item[0]: item[1] for item in obj_dict.__dict__.items()}
            dct2 = {}

            for KEY, VALUE in dct.items():
                if type(VALUE) not in UNIQUE_TYPES:
                    if inspect.isroutine(VALUE):
                        dct2[cls.to_dict(KEY)] = cls.to_dict(VALUE, is_inner_func=True)
                    else:
                        dct2[cls.to_dict(KEY)] = cls.to_dict(VALUE)
            return dct2

        if type(obj) in (int, float, bool, str, nonetype):
            return obj

        if type(obj) in (set, frozenset, tuple, bytes, bytearray):
            return {TYPE_KW: type(obj).__name__,
                    SOURCE_KW: cls.to_dict([*obj])}

        if type(obj) is complex:
            return {TYPE_KW: complex.__name__,
                    SOURCE_KW: {complex.real.__name__: obj.real, complex.imag.__name__: obj.imag}}

        if type(obj) is list:
            return [cls.to_dict(o) for o in obj]

        if type(obj) is dict:
            return {TYPE_KW: dict.__name__,
                    SOURCE_KW: [[cls.to_dict(item[0]), cls.to_dict(item[1])] for item in obj.items()]}

        if type(obj) is moduletype:
            return {TYPE_KW: moduletype.__name__,
                    SOURCE_KW: obj.__name__}

        if type(obj) is codetype:
            code = {TYPE_KW: codetype.__name__}
            source = {}

            for (key, value) in inspect.getmembers(obj):
                if key in CODE_PROPS:
                    source[key] = cls.to_dict(value)

            code.update({SOURCE_KW: source})
            return code

        if type(obj) is celltype:
            return {TYPE_KW: celltype.__name__,
                    SOURCE_KW: cls.to_dict(obj.cell_contents)}

        if type(obj) in (smethodtype, cmethodtype):
            return {TYPE_KW: type(obj).__name__,
                    SOURCE_KW: cls.to_dict(obj.__func__, is_inner_func)}

        if inspect.isroutine(obj):
            source = {CODE_KW: cls.to_dict(obj.__code__)}

            name = obj.__name__
            gvars = {}
            for gvar_name in obj.__code__.co_names:
                if gvar_name in obj.__globals__:

                    if type(obj.__globals__[gvar_name]) is moduletype:
                        gvars[gvar_name] = obj.__globals__[gvar_name]

                    elif inspect.isclass(obj.__globals__[gvar_name]):
                        c = obj.__globals__[gvar_name]
                        if is_inner_func and name in c.__dict__ and obj == c.__dict__[name].__func__:
                            gvars[gvar_name] = c.__name__
                        else:
                            gvars[gvar_name] = c

                    elif gvar_name == obj.__code__.co_name:
                        gvars[gvar_name] = obj.__name__

                    else:
                        gvars[gvar_name] = obj.__globals__[gvar_name]
            source[GLOBALS_KW] = cls.to_dict(gvars)

            source[NAME_KW] = cls.to_dict(obj.__name__)

            source[DEFAULTS_KW] = cls.to_dict(obj.__defaults__)

            source[CLOSURE_KW] = cls.to_dict(obj.__closure__)

            return {TYPE_KW: functype.__name__, SOURCE_KW: source}

        elif inspect.isclass(obj):
            source = {NAME_KW: cls.to_dict(obj.__name__),
                      BASES_KW: cls.to_dict(tuple(b for b in obj.__bases__ if b != object)),
                      DICT_KW: get_obj_dict(obj)}

            return {TYPE_KW: type.__name__, SOURCE_KW: source}

        else:
            source = {CLASS_KW: cls.to_dict(obj.__class__), DICT_KW: get_obj_dict(obj)}

            return {TYPE_KW: OBJECT_KW, SOURCE_KW: source}

    @classmethod
    def from_dict(cls, obj, is_dict=False):
        if is_dict:
            return {cls.from_dict(item[0]): cls.from_dict(item[1]) for item in obj}

        if type(obj) not in (dict, list):
            return obj

        elif type(obj) is list:
            return [cls.from_dict(o) for o in obj]

        else:
            obj_type = obj[TYPE_KW]
            obj_source = obj[SOURCE_KW]

            if obj_type == dict.__name__:
                return cls.from_dict(obj_source, is_dict=True)

            cols_dict = {t.__name__: t for t in [set, frozenset, tuple, bytes, bytearray]}
            if obj_type in cols_dict:
                return cols_dict[obj_type](cls.from_dict(obj_source))

            if obj_type == complex.__name__:
                return obj_source[complex.real.__name__] + \
                    obj_source[complex.imag.__name__] * 1j

            if obj_type == moduletype.__name__:
                return __import__(obj_source)

            if obj_type == codetype.__name__:
                return codetype(*[cls.from_dict(obj_source[prop]) for prop in CODE_PROPS])

            if obj_type == celltype.__name__:
                return celltype(cls.from_dict(obj_source))

            if obj_type == smethodtype.__name__:
                return staticmethod(cls.from_dict(obj_source))

            if obj_type == cmethodtype.__name__:
                return classmethod(cls.from_dict(obj_source))

            if obj_type == functype.__name__:
                code = cls.from_dict(obj_source[CODE_KW])
                gvars = cls.from_dict(obj_source[GLOBALS_KW])
                name = cls.from_dict(obj_source[NAME_KW])
                defaults = cls.from_dict(obj_source[DEFAULTS_KW])
                closure = cls.from_dict(obj_source[CLOSURE_KW])

                for key in gvars:
                    if key in code.co_name and key in globals():
                        gvars[key] = globals()[key]

                func = functype(code, gvars, name, defaults, closure)

                if func.__name__ in gvars:
                    func.__globals__.update({func.__name__: func})
                return func

            if obj_type == type.__name__:
                name = cls.from_dict(obj_source[NAME_KW])
                bases = cls.from_dict(obj_source[BASES_KW])
                dct = obj_source[DICT_KW]
                dct = {cls.from_dict(item[0]): cls.from_dict(item[1]) for item in dct.items()}

                cl = type(name, bases, dct)

                for attr in cl.__dict__.values():
                    if inspect.isroutine(attr):
                        if type(attr) in (smethodtype, classmethod):
                            fglobs = attr.__func__.__globals__
                        else:
                            fglobs = attr.__globals__

                        for gv in fglobs.keys():
                            if gv == cl.__name__:
                                fglobs[gv] = cl
                return cl

            else:
                clas = cls.from_dict(obj_source[CLASS_KW])
                dct = obj_source[DICT_KW]
                dct = {cls.from_dict(item[0]): cls.from_dict(item[1]) for item in dct.items()}

                o = object.__new__(clas)
                o.__dict__ = dct
                return o



from abc import ABC, abstractmethod


class Serializer(ABC):
    @abstractmethod
    def dumps(self, obj) -> str:
        pass

    @abstractmethod
    def loads(self, string: str):
        pass

    def dump(self, obj, path: str):
        with open(path, "w") as file:
            file.write(self.dumps(obj))

    def load(self, path):
        with open(path, "r") as file:
            return self.loads(file.read())



import regex
from kolyaklimkLab3.MySerializer import Serializer
from kolyaklimkLab3.MyParser import Parser, nonetype
from kolyaklimkLab3.MyParser.XmlParser import KEY_GROUP_NAME, VALUE_GROUP_NAME, XML_SCHEME_SOURCE, XML_ELEMENT_PATTERN, \
    FIRST_XML_ELEMENT_PATTERN


class Xml(Serializer):
    def dumps(self, obj) -> str:
        def create_xml_element(name: str, data: str, is_first=False):
            if is_first:
                return f"<{name} {XML_SCHEME_SOURCE}>{data}</{name}>"
            else:
                return f"<{name}>{data}</{name}>"

        def dumps_from_dict(string, is_first=False) -> str:
            if type(string) in (int, float, bool, nonetype):
                return create_xml_element(type(string).__name__, str(string), is_first)

            if type(string) is str:
                data = string.replace('&', "&amp;").replace('<', "&lt;").replace('>', "&gt;"). \
                    replace('"', "&quot;").replace("'", "&apos;")
                return create_xml_element(str.__name__, data, is_first)

            if type(string) is list:
                data = ''.join([dumps_from_dict(o) for o in string])
                return create_xml_element(list.__name__, data, is_first)

            if type(string) is dict:
                data = ''.join([f"{dumps_from_dict(item[0])}{dumps_from_dict(item[1])}" for item in string.items()])
                return create_xml_element(dict.__name__, data, is_first)
            else:
                raise ValueError

        obj = Parser.to_dict(obj)
        return dumps_from_dict(obj, True)

    def loads(self, obj: str):
        def loads_to_dict(string: str, is_first=False):
            string = string.strip()
            xml_element_pattern = FIRST_XML_ELEMENT_PATTERN if is_first else XML_ELEMENT_PATTERN

            match = regex.fullmatch(xml_element_pattern, string)

            if not match:
                raise ValueError

            key = match.group(KEY_GROUP_NAME)
            value = match.group(VALUE_GROUP_NAME)

            if key == int.__name__:
                return int(value)

            if key == float.__name__:
                return float(value)

            if key == bool.__name__:
                return value == str(True)

            if key == str.__name__:
                return value.replace("&amp;", '&').replace("&lt;", '<').replace("&gt;", '>'). \
                    replace("&quot;", '"').replace("&apos;", "'")

            if key == nonetype.__name__:
                return None

            if key == list.__name__:
                matches = regex.findall(XML_ELEMENT_PATTERN, value)
                return [loads_to_dict(match[0]) for match in matches]

            if key == dict.__name__:
                matches = regex.findall(XML_ELEMENT_PATTERN, value)
                return {loads_to_dict(matches[i][0]): loads_to_dict(matches[i + 1][0]) for i in
                        range(0, len(matches), 2)}
            else:
                raise ValueError

        obj = loads_to_dict(obj, True)
        return Parser.from_dict(obj)



from types import \
    NoneType as nonetype, \
    ModuleType as moduletype, \
    CodeType as codetype, \
    FunctionType as functype, \
    BuiltinFunctionType as bldinfunctype, \
    CellType as celltype, \
    MappingProxyType as mapproxytype, \
    WrapperDescriptorType as wrapdesctype, \
    MethodDescriptorType as metdesctype, \
    GetSetDescriptorType as getsetdesctype

CODE_PROPS = [prop.__name__ for prop in [
    codetype.co_argcount,
    codetype.co_posonlyargcount,
    codetype.co_kwonlyargcount,
    codetype.co_nlocals,
    codetype.co_stacksize,
    codetype.co_flags,
    codetype.co_code,
    codetype.co_consts,
    codetype.co_names,
    codetype.co_varnames,
    codetype.co_filename,
    codetype.co_name,
    #codetype.co_qualname,
    codetype.co_firstlineno,
    codetype.co_lnotab,
    #codetype.co_exceptiontable,
    codetype.co_freevars,
    codetype.co_cellvars
]]

UNIQUE_TYPES = [
    mapproxytype,
    wrapdesctype,
    metdesctype,
    getsetdesctype,
    bldinfunctype
]

TYPE_KW = "type"
SOURCE_KW = "source"
CODE_KW = "__code__"
GLOBALS_KW = functype.__globals__.__name__
NAME_KW = "__name__"
DEFAULTS_KW = "__defaults__"
CLOSURE_KW = functype.__closure__.__name__
BASES_KW = "__bases__"
DICT_KW = "__dict__"
CLASS_KW = "__class__"
OBJECT_KW = "object"


class static_class:
    @staticmethod
    def sm():
        pass

    @classmethod
    def cm(cls):
        pass


smethodtype = type(static_class.__dict__["sm"])
cmethodtype = type(static_class.__dict__["cm"])



from enum import Enum

from kolyaklimkLab3.MyParser.XmlParser.Xml import Xml
from kolyaklimkLab3.MyParser.JsonParser.Json import Json


class SerializerType(Enum):
    JSON = "json"
    XML = "xml"


class Factory:
    @staticmethod
    def create_serializer(st: SerializerType):
        if st == SerializerType.JSON:
            return Json()
        elif st == SerializerType.XML:
            return Xml()
        else:
            raise Exception("Unknown type")



# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

import kolyaklimkLab3

setup(
    name="kolyaklimkLab3",
    version="4.0.0",
    description='',
    author_email="kolyaklimk@gmail.com",
    packages=find_packages(include=["kolyaklimkLab3", "kolyaklimkLab3.*"]),
    zip_safe=False
)



from kolyaklimkLab3.MyParser.Constants import nonetype, moduletype, codetype, celltype, functype, bldinfunctype, \
    smethodtype, \
    cmethodtype, mapproxytype, wrapdesctype, metdesctype, getsetdesctype, CODE_PROPS, UNIQUE_TYPES, TYPE_KW, SOURCE_KW, \
    CODE_KW, GLOBALS_KW, NAME_KW, DEFAULTS_KW, CLOSURE_KW, BASES_KW, DICT_KW, CLASS_KW, OBJECT_KW

from kolyaklimkLab3.MyParser.Parser import Parser
